﻿// Demo.AdminApi
//
// Copyright © 2018 Eurostep
//
// All rights reserved.
//
// MIT License
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
// the Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;

namespace Demo.AdminApi
{
    public static partial class Program
    {
        public static void ChangePassword(string baseAddress, string authApiKey, string email, string password, string uoiVersion)
        {
            byte[] bytes = Encoding.Unicode.GetBytes(password);
            SHA512Managed sha = new SHA512Managed();
            byte[] hash = sha.ComputeHash(bytes);
            sha.Dispose();
            string passwordHash = string.Empty;
            foreach (byte b in hash)
            {
                passwordHash += string.Format("{0:x2}", b);
            }
            string relativeUri = $"/identity/user/updatePassword?email={email}&password={passwordHash}";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(authApiKey, relativeUri);
            client.DefaultRequestHeaders.Add("SAs-UOIVersion", uoiVersion);
            HttpResponseMessage response = client.PutAsync(relativeUri, null).Result;
            response.EnsureSuccessStatusCode();
            client.Dispose();
        }

        public static void ReIndex(string baseAddress, string adminApiKey, string spaceId)
        {
            string relativeUri = $"/admin/reindex/{spaceId}";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(adminApiKey, relativeUri);
            HttpResponseMessage response = client.PostAsync(relativeUri, null).Result;
            response.EnsureSuccessStatusCode();
            client.Dispose();
        }

        public static void RebuildAnIndex(string baseAddress, string adminApiKey, string spaceId, string indexId)
        {
            string relativeUri = $"/admin/reindex/{spaceId}/{indexId}";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(adminApiKey, relativeUri);
            HttpResponseMessage response = client.PostAsync(relativeUri, null).Result;
            response.EnsureSuccessStatusCode();
            client.Dispose();
        }

        public static void GetSnapshot(string baseAddress, string snapshotApiKey, string filePath)
        {
            string relativeUri = $"/admin/snapshot";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(snapshotApiKey, relativeUri);
            Stream response = client.GetStreamAsync(relativeUri).Result;
            using (FileStream file = new FileStream(filePath, FileMode.Create, FileAccess.Write))
            {
                response.CopyTo(file);
            }
            client.Dispose();
        }

        public static void InstallSnapshot(string baseAddress, string snapshotApiKey, string filePath)
        {
            string relativeUri = $"/system/snapshot";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(snapshotApiKey, relativeUri);
            using (FileStream file = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                StreamContent content = new StreamContent(file);
                HttpResponseMessage response = client.PostAsync(relativeUri, content).Result;
                response.EnsureSuccessStatusCode();
            }
            client.Dispose();
        }

        public static void Upload(string baseAddress, string taskApiKey, string path, string filePath)
        {
            string relativeUri = $"/system/upload/{path}";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(taskApiKey, relativeUri);
            using (FileStream file = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                StreamContent content = new StreamContent(file);
                HttpResponseMessage response = client.PostAsync(relativeUri, content).Result;
                response.EnsureSuccessStatusCode();
            }
            client.Dispose();
        }

        public static void Download(string baseAddress, string taskApiKey, string path, string filePath)
        {
            string relativeUri = $"/system/download/{path}";
            HttpClient client = new HttpClient();
            client.BaseAddress = new Uri(baseAddress);
            client.DefaultRequestHeaders.Authorization = new BearerAuthenticationHeaderValue(taskApiKey, relativeUri);
            Stream response = client.GetStreamAsync(relativeUri).Result;
            using (FileStream file = new FileStream(filePath, FileMode.Create, FileAccess.Write))
            {
                response.CopyTo(file);
            }
            client.Dispose();
        }
    }
}